userprog: move to 'test/' directory
[model-checker.git] / main.cc
diff --git a/main.cc b/main.cc
index 5d036f2d5c940350633f98defb5165d4cba63704..73b3b0c410d95fbdbd06190eadd35b25b1288e77 100644 (file)
--- a/main.cc
+++ b/main.cc
@@ -1,18 +1,24 @@
+/** @file main.cc
+ *  @brief Entry point for the model checker.
+ */
+
 #include "libthreads.h"
 #include "common.h"
 #include "threads.h"
 
+#include "datarace.h"
+
 /* global "model" object */
 #include "model.h"
-#include "snapshot.h"
 #include "snapshot-interface.h"
 
-/*
- * Return 1 if found next thread, 0 otherwise
+/**
+ * The thread_system_next function takes the next step in the execution.
+ * @return Returns the 1 if there is another step and 0 otherwise.
  */
 static int thread_system_next(void) {
        Thread *curr, *next;
-  
+
        curr = thread_current();
        if (curr) {
                if (curr->get_state() == THREAD_READY) {
@@ -25,6 +31,8 @@ static int thread_system_next(void) {
                        ASSERT(false);
        }
        next = model->scheduler->next_thread();
+       if (!model->isfeasible())
+               return 1;
        if (next)
                next->set_state(THREAD_RUNNING);
        DEBUG("(%d, %d)\n", curr ? curr->get_id() : -1, next ? next->get_id() : -1);
@@ -33,44 +41,54 @@ static int thread_system_next(void) {
        return Thread::swap(model->get_system_context(), next);
 }
 
+/** The thread_wait_finish method runs the current execution until we
+ *  have no more steps to take.
+ */
 static void thread_wait_finish(void) {
        DBG();
 
        while (!thread_system_next());
 }
 
-void real_main() {
+
+/** The real_main function contains the main model checking loop. */
+static void real_main() {
        thrd_t user_thread;
        ucontext_t main_context;
 
-       //Create the singleton snapshotStack object
-       snapshotObject = new snapshotStack();
-  
+       //Initialize race detector
+       initRaceDetector();
+
+       //Create the singleton SnapshotStack object
+       snapshotObject = new SnapshotStack();
+
        model = new ModelChecker();
-  
+
        if (getcontext(&main_context))
                return;
-  
+
        model->set_system_context(&main_context);
 
+       snapshotObject->snapshotStep(0);
        do {
                /* Start user program */
                model->add_thread(new Thread(&user_thread, (void (*)(void *)) &user_main, NULL));
-    
+
                /* Wait for all threads to complete */
                thread_wait_finish();
        } while (model->next_execution());
-  
+
        delete model;
-  
+
        DEBUG("Exiting\n");
 }
 
 int main_numargs;
 char ** main_args;
 
-/*
- * Main system function
+/**
+ * Main function.  Just initializes snapshotting library and the
+ * snapshotting library calls the real_main function.
  */
 int main(int numargs, char ** args) {
        /* Stash this stuff in case someone wants it eventually */
@@ -78,5 +96,5 @@ int main(int numargs, char ** args) {
        main_args=args;
 
        /* Let's jump in quickly and start running stuff */
-       initSnapShotLibrary(10000 /*int numbackingpages*/, 1024 /*unsigned int numsnapshots*/, 1024 /*unsigned int nummemoryregions*/ , 1000 /*int numheappages*/, &real_main /*MyFuncPtr entryPoint*/);
+       initSnapShotLibrary(10000, 1024, 1024, 1000, &real_main);
 }